@@ -31,10 +31,20 @@ class SanicConf(BaseModel):
     # reload_dir: Optional[Union[List[str], str]] = None
 
 
+class ServingPaths(BaseModel):
+    uri: str
+    path: str
+    route: bool = False
+
+
+SanicStaticServing = dict[str, list[ServingPaths]] | None
+
+
 class AppConf(BaseModel):
     app_name: str
     database: DbConfig | None
     sanic: SanicConf | None
+    serving: SanicStaticServing
 
 
 GenericConfig = TypeVar("GenericConfig", bound=BaseModel)
