@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import asyncio
+import copy
 import datetime
 import os
 import uuid
@@ -25,14 +26,48 @@ from web_foundation.kernel.messaging.channel import IMessage
 BackgroundTask = TypeVar("BackgroundTask", Callable[..., Coroutine[Any, Any, Any]], partial)
 
 
+class TaskIMessage(IMessage):
+    message_type = "background_task"
+    task: BackgroundTask
+    args: Any | None
+    kwargs: Any | None
+    trigger: BaseTrigger
+    add_job_kw: Dict
+    on_error_callback: TaskErrorCallback
+    on_complete_callback: TaskCompleteCallback
+    return_event: bool
+    destination = "__dispatcher__"
+
+    def __init__(self, task: BackgroundTask,
+                 trigger=None,
+                 args: Any = None,
+                 kwargs: Any | None = None,
+                 add_job_kw: Dict = None,
+                 on_error_callback: TaskErrorCallback = None,
+                 on_complete_callback: TaskCompleteCallback = None,
+                 return_event: bool = False):
+        super().__init__()
+        self.task = task
+        self.args = args
+        self.kwargs = kwargs
+        self.add_job_kw = add_job_kw if add_job_kw else {}
+        self.trigger = trigger
+        self.on_error_callback = on_error_callback
+        self.on_complete_callback = on_complete_callback
+        self.return_event = return_event
+
+
 @dataclass
 class Task:
     id: str
     status: str
     name: str
     scheduled_trigger: BaseTrigger
+    trigger_message: TaskIMessage
+    result: Any | None = field(default=None)
     scheduled_job: Job | None = field(default=None)
     on_error_callback: TaskErrorCallback = field(default=None)
+    on_complete_callback: TaskCompleteCallback = field(default=None)
     next_call_time: datetime.datetime | None = field(default=None)
     execution_time: float = field(default=0)
     done_time: float = field(default=0)
@@ -43,38 +78,19 @@ class Task:
     return_event: bool = False
 
 
-TaskErrorCallback = Callable[[JobExecutionEvent, Task], Coroutine[Any, Any, None]]
+class TaskIMessageReturn(IMessage):
+    task: Task
+    message_type = "background_task_return"
 
-
-class TaskIMessage(IMessage):
-    message_type = "background_task"
-    task: BackgroundTask
-    args: Any | None
-    kwargs: Any | None
-    trigger: BaseTrigger
-    add_job_kw: Dict
-    on_error_callback: TaskErrorCallback
-    return_event: bool
-    destination = "__dispatcher__"
-
-    def __init__(self, task: BackgroundTask,
-                 trigger=None,
-                 args: Any = None,
-                 kwargs: Any | None = None,
-                 add_job_kw: Dict = None,
-                 on_error_callback: TaskErrorCallback = None,
-                 return_event: bool = False):
+    def __init__(self, task: Task):
         super().__init__()
         self.task = task
-        self.args = args
-        self.kwargs = kwargs
-        self.add_job_kw = add_job_kw if add_job_kw else {}
-        self.trigger = trigger
-        self.on_error_callback = on_error_callback
-        self.return_event = return_event
+        self.destination = self.task.trigger_message.sender
 
 
-ProduceCall = Callable[[TaskIMessage], Coroutine[Any, Any, None]]
+TaskErrorCallback = Callable[[JobExecutionEvent, Task], Coroutine[Any, Any, None]]
+TaskCompleteCallback = Callable[[JobExecutionEvent, Task], Coroutine[Any, Any, None]]
+ProduceCall = Callable[[IMessage], Coroutine[Any, Any, None]]
 
 
 class TaskScheduler:
@@ -108,6 +124,7 @@ class TaskScheduler:
         if isinstance(message.trigger, IntervalTrigger):
             for task_id, task in self._tasks.items():
                 if task.scheduled_job.func == message.task:
+                    task.trigger_message = message
                     task.scheduled_job.trigger = message.trigger
                     task.scheduled_trigger = message.trigger
                     task.scheduled_job.args = message.args
@@ -119,7 +136,9 @@ class TaskScheduler:
                         status="new",
                         scheduled_trigger=message.trigger,
                         on_error_callback=message.on_error_callback,
-                        return_event=message.return_event)
+                        on_complete_callback=message.on_complete_callback,
+                        return_event=message.return_event,
+                        trigger_message=message)
         arc_task.scheduled_job = self._scheduler.add_job(message.task,
                                                          name=arc_task.name,
                                                          id=arc_task.id,
@@ -140,20 +159,25 @@ class TaskScheduler:
             task.call_counter += 1
             task.status = "called"
 
-        def _on_job_exec(event: JobExecutionEvent):  # todo remove done jobs?
+        def _on_job_exec(event: JobExecutionEvent):  # todo remove done jobs? YAROHA: NO, NEED TO STATISTICS
             nonlocal self
             task = self._tasks.get(event.job_id)
             task.done_time = time_ns()
             task.execution_time = task.done_time - task.call_time
             task.status = "done"
             task.next_call_time = task.scheduled_job.next_run_time
+            task.result = event.retval
             if event.exception:
                 task.error = event.exception
                 task.status = 'error'
                 if task.on_error_callback:
                     asyncio.create_task(task.on_error_callback(event, task))
-            elif task.return_event and event.retval:
-                asyncio.create_task(self.produce_method(event.retval))
+            else:
+                if task.on_complete_callback:
+                    asyncio.create_task(task.on_complete_callback(event, task))
+                if task.return_event:
+                    ret_event = TaskIMessageReturn(task)
+                    asyncio.create_task(self.produce_method(ret_event))
 
         self._scheduler.add_listener(_on_job_submitted, EVENT_JOB_SUBMITTED)
         self._scheduler.add_listener(_on_job_exec, EVENT_JOB_EXECUTED | EVENT_JOB_ERROR)
