@@ -8,10 +8,11 @@ from inspect import isawaitable, iscoroutine, iscoroutinefunction
 from pathlib import Path
 from typing import Union, List, Type, Callable, Coroutine, Any
 
+import loguru
+
 from web_foundation.config import GenericConfig
 from web_foundation.errors.app.application import InconsistencyError
 
-
 # ------------------------------------------------STR_TO---------------------------------------------------------------
 from web_foundation.kernel.dependency import Dependency
 
@@ -113,7 +114,8 @@ def in_obj_subcls_of_dependencies(obj: object | Type, target_cls: Type):
             if not i.startswith("__"):
                 item = getattr(obj, i)
                 if isinstance(item, Dependency):
-                    item = item.initialized
+                    test = item
+                    item = item()
                 if item.__class__ in all_subclasses(target_cls):
                     yield item, i
     except Exception as e:
